// *******************************************************************
// Broker.h
// *******************************************************************
//
//  DESCRIPTION: API for Broker class. Used by clients of the
//                        DocConversion project to broker a conversion
//                        of a given document.
//  NOTE:             This program makes use of strings and Document.h.
//
/***************************************************************************
 *   Copyright (C) 2003 by drs. Eric D. Schabell                           *
 *   erics@cs.kun.nl                                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef _BROKER_H_
#define _BROKER_H_

//--------------------------------------------------------------------
//#includes
//--------------------------------------------------------------------
#include <string>           // STL strings.
#include <pqxx/all.h>       // postgresql api.
#include "Document.h"

using namespace std;
using namespace pqxx;


class Broker
{

public:

    //--------------------------------------------------------------------
    //Purpose:           default constructor.
    //Preconditions:    none.
    //Postconditions:  a Broker object is instantiated.
    //Arguments:       none.
    //Returns:           Broker.
    //Called Funcs:    increaseCounter().
    //--------------------------------------------------------------------
    Broker ();

    //--------------------------------------------------------------------
    //Purpose:            constructor with a (remote) server name given.
    //Preconditions:    must supply a valid server hostname.
    //Postconditions:  a Broker object is instantiated with a
    //                       remote server hostname.
    //Arguments:       string name.
    //Returns:           Broker.
    //Called Funcs:    increaseCounter().
    //--------------------------------------------------------------------
    Broker ( const string name );

    //--------------------------------------------------------------------
    //Purpose:            default destructor.
    //Preconditions:    Broker object.
    //Postconditions:  Broker object destroyed.
    //Arguments:       none.
    //Returns:           0
    //Called Funcs:    decreaseCounter().
    //--------------------------------------------------------------------
    ~Broker();

    //--------------------------------------------------------------------
    //Purpose:            obtain the Broker Counter's current value.
    //Preconditions:    none.
    //Postconditions:  returns brokerCounter value.
    //Arguments:       none.
    //Returns:           const int brokerCounter.
    //Called Funcs:    none.
    //--------------------------------------------------------------------
    const int getCounter();

    //--------------------------------------------------------------------
    //Purpose:            obtain the name of the server which the Broker
    //                        is brokering with.
    //Preconditions:    none.
    //Postconditions:  returns serverName.
    //Arguments:       none.
    //Returns:           const string serverName.
    //Called Funcs:    none.
    //--------------------------------------------------------------------
    const string getServerName();

    //--------------------------------------------------------------------
    //Purpose:            prepare for conversion of the provided Document,
    //                        and pass to Server for actual converison.
    //Preconditions:    Document object.
    //Postconditions:  returns true if conversion successfull, false if not.
    //                       Converted document location is registered in the
    //                       Document.conLocation attribute.
    //Arguments:       Document& doc.
    //Returns:           bool.
    //Called Funcs:   Document.getDocumentConversion(),
    //                      getServerName(), Server(),
    //                      Server.requestConversion().
    //--------------------------------------------------------------------
    bool convertDocument ( Document& doc );

    //--------------------------------------------------------------------
    //Purpose:          obtain list of conversions in conversion database.
    //Preconditions:    Server must be available.
    //Postconditions:   returns Result object contianing the list of available
    //                        conversions.
    //                       This is a 5 column answer:
    //                         routine_name,
    //                         from_feature_type,
    //                         to_feature_type,
    //                         from_representation_type,
    //                         to_representation_type.
    //Arguments:        none.
    //Returns:          Result.
    //Called Funcs:     Server.listConversionDB().
    //--------------------------------------------------------------------
    Result getConversionListing();


    //--------------------------------------------------------------------
    //Purpose:          obtain list of available conversions in conversion
    //                      database based on given representation type..
    //Preconditions:    Server must be available.
    //Postconditions:   returns Result object contianing the list of available
    //                        conversions.
    //                       This is a 5 column answer:
    //                         routine_name,
    //                         from_feature_type,
    //                         to_feature_type,
    //                         from_representation_type,
    //                         to_representation_type.
    //Arguments:        none.
    //Returns:          Result.
    //Called Funcs:     Broker(), Server.listRepresentaionTypes( string representationType ).
    //--------------------------------------------------------------------
    Result getRepresentationTypeListing( string representationType );

    //--------------------------------------------------------------------
    //Purpose:          obtain list of available conversions in conversion
    //                      database based on given feature type..
    //Preconditions:    Server must be available.
    //Postconditions:   returns Result object contianing the list of available
    //                        conversions.
    //                       This is a 5 column answer:
    //                         routine_name,
    //                         from_feature_type,
    //                         to_feature_type,
    //                         from_representation_type,
    //                         to_representation_type.
    //Arguments:        none.
    //Returns:          Result.
    //Called Funcs:     Broker(), Server.listFeatureTypes( string featureType ).
    //--------------------------------------------------------------------
    Result getFeatureTypeListing( string featureType );



private:
    static int brokerCounter;   // keeps track of  nr. activerokers.
    static const string DEFAULT_NAME;  // localhost used if notinitialized.
    string serverName;          // name of conversion server..

    //--------------------------------------------------------------------
    //Purpose:            to increase the brokerCounter attribute.
    //Preconditions:    none.
    //Postconditions:  brokerCounter attribute has increased by one.
    //Arguments:       none.
    //Returns:           0.
    //Called Funcs:    none.
    //--------------------------------------------------------------------
    void increaseCounter();

    //--------------------------------------------------------------------
    //Purpose:            to decrease the brokerCounter attribute.
    //Preconditions:    none.
    //Postconditions:  brokerCounter attribute has decreased by one.
    //Arguments:       none.
    //Returns:           0.
    //Called Funcs:    none.
    //--------------------------------------------------------------------
    void decreaseCounter();
};

#endif /* _BROKER_H_ */
