// *******************************************************************
// Document.cpp
// *******************************************************************
//
//  DESCRIPTION: API for Document class. Used to manipulate a
//                         given document.
//  NOTE:             This program makes use of strings and the
//                        basename() system call.
//
/***************************************************************************
*   Copyright (C) 2003 by drs. Eric D. Schabell                           *
*   erics@cs.kun.nl                                                       *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
***************************************************************************/

//--------------------------------------------------------------------
//#includes
//--------------------------------------------------------------------
#include <iostream>
#include <string>
#include "Document.h"

using namespace std;

//--------------------------------------------------------------------
//Purpose:           default constructor.
//Preconditions:    none.
//Postconditions:  a Document object is instantiated and all
//                       attributes are zeroed.
//Arguments:       none.
//Returns:           Document.
//Called Funcs:    none.
//--------------------------------------------------------------------
Document::Document ()
{
    url = "0";
    file = "0";
    conLocation = "0";
    fileLocation = "0";
}

//--------------------------------------------------------------------
//Purpose:            constructor with url and conversion given..
//Preconditions:    must supply a string document url and a
//                        string conversion routine.
//Postconditions:  a Document object is instantiated with a
//                       document url, file name and conversion
//                       routine defined. File location and Conversion
//                       location are zeroed.
//Arguments:       string doc_url, string routine.
//Returns:           Document.
//Called Funcs:    setDocumentFile(), setDocumentConversion().
//--------------------------------------------------------------------
Document::Document(string doc_url, string routine)
{
    // setting url and file now.
    url = doc_url;
    setDocumentFile();
    setDocumentConversion(routine);

    // prepare fileLocation and conLocation for latter assignment.
    fileLocation = "0";
    conLocation = "0";
}

//--------------------------------------------------------------------
//Purpose:            default destructor.
//Preconditions:    Document object.
//Postconditions:  Document object destroyed.
//Arguments:       none.
//Returns:           0
//Called Funcs:    none.
//--------------------------------------------------------------------
Document::~Document()
{
    // cout << "Destroying Docuemnt object. << endl;
}

//--------------------------------------------------------------------
//Purpose:            obtain the docuemnts complete url.
//Preconditions:    Docuemnt object exists.
//Postconditions:  returns url attribute.
//Arguments:       none.
//Returns:           string.
//Called Funcs:    none.
//--------------------------------------------------------------------
string Document::getDocumentUrl()
{
    return url;
}

//--------------------------------------------------------------------
//Purpose:            Return the file name if instantiated, otherwise
//                        returns the url attribute.
//Preconditions:    Docuemnt object created.
//Postconditions:  success = returns instantiated file attribute.
//                       failure    = returns url if file has not been instantiated,
//                                      which signals a problem with the
//                                      document construction.
//Arguments:       none.
//Returns:           string
//Called Funcs:    none.
//--------------------------------------------------------------------
string Document::getDocumentFile()
{
    if (file == "0")
    {
        cout << "Error: file not initialized yet?" << endl;
        // returning url instead to provide at least some sort of
        // file to the caller.
        return url;
    }
    else
    {
        // file is really only a file name.
        return file;
    }
}

//--------------------------------------------------------------------
//Purpose:            Return the conversion routine attribute.
//Preconditions:    Docuemnt object created.
//Postconditions:  returns instantiated file attribute.
//Arguments:       none.
//Returns:           string
//Called Funcs:    none.
//--------------------------------------------------------------------
string Document::getDocumentConversion()
{
    if (conversion == "0")
    {
        cout << "Error: conversion has not been initialized yet?" << endl;
        // return the NULL value for user.
        return conversion;
    }
    else
    {
        // conversion is initialized, return.
        return conversion;
    }
}

//--------------------------------------------------------------------
//Purpose:            Return the conLocation attribute.
//Preconditions:    Docuemnt object created.
//Postconditions:  returns conLocation attribute.
//Arguments:       none.
//Returns:           string
//Called Funcs:    none.
//--------------------------------------------------------------------
string Document::getConversionLocation()
{
    if (conLocation == "0")
    {
        cout << "Error: conLocation has not been initialized yet?" << endl;
        // return the NULL value for user.
        return conLocation;
    }
    else
    {
        // conLocation is initialized, return.
        return conLocation;
    }
}

//--------------------------------------------------------------------
//Purpose:            Return the docuemnt file location.
//Preconditions:    Docuemnt object created.
//Postconditions:  returns fileLocation attribute.
//Arguments:       none.
//Returns:           string
//Called Funcs:    none.
//--------------------------------------------------------------------
string Document::getFileLocation()
{
    return fileLocation;
}

//--------------------------------------------------------------------
//Purpose:            Determine and set the file name by basnaming
//                        the url attribute.
//Preconditions:    Docuemnt object created.
//Postconditions:  success = a set file attribute.
//                       failure    = returns error
//Arguments:       none.
//Returns:           0.
//Called Funcs:    basename().
//--------------------------------------------------------------------
void Document::setDocumentFile()
{
    // attempt to determine basename() with Linux system call.
    if ( basename( url.c_str() ) )
    {
        file = basename(url.c_str() );
        return;
    }
    else
    {
        cerr << "File name could not be determined with basename()." << endl;
        exit(1);
    }
}

//--------------------------------------------------------------------
//Purpose:           Instantiates the conversion attribute with the
//                       given convertion routine.
//Preconditions:   Docuemnt object created, convertion routine
//                       provided. .
//Postconditions:  instantiated conversion attribute.
//Arguments:       string convertion_request..
//Returns:           0.
//Called Funcs:    none.
//--------------------------------------------------------------------
void Document::setDocumentConversion(string conversion_request)
{
    // set with passed parameter, no checking yet.
    conversion = conversion_request;
}

//--------------------------------------------------------------------
//Purpose:            Sets the converted files location to given location.
//Preconditions:    Docuemnt object created, location provided.
//Postconditions:  conLocation attribute is set with given file
//                       location.
//Arguments:       string conversion_location.
//Returns:           0.
//Called Funcs:    none.
//--------------------------------------------------------------------
void Document::setConversionLocation(string conversion_location)
{
    conLocation = conversion_location;
}

//--------------------------------------------------------------------
//Purpose:            Sets the file location to given location.
//Preconditions:    Docuemnt object created, location provided.
//Postconditions:  fileLocation attribute is set with given file
//                       location.
//Arguments:       string file_location.
//Returns:           0.
//Called Funcs:    none.
//--------------------------------------------------------------------
void Document::setFileLocation(string file_location)
{
    fileLocation = file_location;
}
