// *******************************************************************
// Server.h
// *******************************************************************
//
//  DESCRIPTION:  contains the function prototypes and the #includes
//                         used in implementation of the DocConversion Server.
//
//  NOTE:              This program makes use of the STL string.
//
//  CHANGE LOG:
//                         2003-10-15  Initial creation.
//
/***************************************************************************
 *   Copyright (C) 2003 by drs. Eric D. Schabell                           *
 *   erics@cs.kun.nl                                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

//--------------------------------------------------------------------
//conditional compilation
//--------------------------------------------------------------------
#ifndef _SERVER_H
#define _SERVER_H


//--------------------------------------------------------------------
//#includes
//--------------------------------------------------------------------
#include <string>
#include "Document.h"
#include <pqxx/all.h>       // postgresql api.

using namespace std;
using namespace pqxx;



class Server
{
public:

    //--------------------------------------------------------------------
    //Purpose:           Default constructor.
    //Preconditions:    none.
    //Postconditions:  Server object created.
    //Arguments:        none.
    //Returns:            0.
    //Called Funcs:    none.
    //--------------------------------------------------------------------
    Server();

    //--------------------------------------------------------------------
    //Purpose:           Constructo with server address.
    //Preconditions:    none..
    //Postconditions:  Server object created, serverAddress set.
    //Arguments:        string address.
    //Returns:            0.
    //Called Funcs:    none.
    //--------------------------------------------------------------------
    Server( string address );

    //--------------------------------------------------------------------
    //Purpose:           Destructor.
    //Preconditions:    Server object exists..
    //Postconditions:  Server object destroyed.
    //Arguments:        none.
    //Returns:            0.
    //Called Funcs:    none.
    //--------------------------------------------------------------------
    ~Server();

    //--------------------------------------------------------------------
    //Purpose:            conversion of the provided Document, based
    //                        on the objects registered conversion routine.
    //Preconditions:    Document object.
    //Postconditions:  returns true if conversion successfull, false if not.
    //                       Converted document location is registered in the
    //                       Document.conLocation attribute.
    //Arguments:       Document& doc.
    //Returns:           bool.
    //Called Funcs:   Document.getDocumentConversion(),
    //                      Document.getFileLocation(), getFile(),
    //                      Document.getDocumentFile(),
    //                      Document.setFileLocation(),
    //                      Document.getDocumentConversion(), system(),
    //                      c_str(), Document.setConversionLocation().
    //--------------------------------------------------------------------
    bool requestConversion( Document& doc );

    //--------------------------------------------------------------------
    //Purpose:          obtain list of conversions in database.
    //Preconditions:    DB must be available.
    //Postconditions:   returns Result object contianing the list of available
    //                        conversions.
    //                       This is a 5 column answer:
    //                         routine_name,
    //                         from_feature_type,
    //                         to_feature_type,
    //                         from_representation_type,
    //                         to_representation_type.
    //Arguments:        none.
    //Returns:          Result.
    //Called Funcs:     Connection(), Transaction(), Result(),
    //                        Transaction.Exec(), Transaction.Commit().
    //--------------------------------------------------------------------
    Result listConversionDB();

    //--------------------------------------------------------------------
    //Purpose:          obtain list of conversions in database based on
    //                      given representation type.
    //Preconditions:   DB must be available.
    //Postconditions: returns Result object contianing the list of available
    //                      conversions.
    //                      This is a 5 column answer:
    //                      routine_name,
    //                      from_feature_type,
    //                      to_feature_type,
    //                      from_representation_type,
    //                      to_representation_type.
    //Arguments:       none.
    //Returns:           Result.
    //Called Funcs:    Connection(), Transaction(), Result(),
    //                       Transaction.Exec(), Transaction.Commit().
    //--------------------------------------------------------------------
    Result listRepesentationTypes( string representationType );


    //--------------------------------------------------------------------
    //Purpose:          obtain list of conversions in database based on
    //                      given feature type.
    //Preconditions:   DB must be available.
    //Postconditions: returns Result object contianing the list of available
    //                      conversions.
    //                      This is a 5 column answer:
    //                      routine_name,
    //                      from_feature_type,
    //                      to_feature_type,
    //                      from_representation_type,
    //                      to_representation_type.
    //Arguments:       none.
    //Returns:           Result.
    //Called Funcs:    Connection(), Transaction(), Result(),
    //                       Transaction.Exec(), Transaction.Commit().
    //--------------------------------------------------------------------
    Result listFeatureTypes( string featureType );


private:
    string serverAddress;                     // address of server, either dns name or ip.
    const string DEFAULT_ADDRESS;  // localhost used if notinitialized.

    //--------------------------------------------------------------------
    //Purpose:            retrieve the Document file from its url location.
    //Preconditions:    Document object.
    //Postconditions:  returns true if file retrieval successfull, false if not.
    //                       Only does remote retrieval after checking for local
    //                       presence of the file. The location of the file is set
    //                       for the Document.fileLocation attribute.
    //Arguments:       Document& doc.
    //Returns:           bool.
    //Called Funcs:   Document.getDocumentFile(), ifstream.open(),
    //                      c_str(), system(), Document.getDocumentUrl(),
    //                      Document.setFileLocation(), ifstream.close().
    //--------------------------------------------------------------------
    bool getFile ( Document& doc );
};

#endif
